/*
 Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 
 Author: Sebastian Scholz <sebastian.scholz@aizo.com>
 
 This file is part of Auto-Off digitalSTROM Server add on.
 Auto-Off is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 Auto-Off is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */


//= require <gettexthelper>
Ext.namespace('SceneResponder', 'SceneResponder.config');

var params = 
{
    "domain": "scene-responder",
    "locale_data": json_locale_data
};

SceneResponder.config.gt = new Gettext(params);

SceneResponder.config.VERSION = "0.1.1"

function _(msgid)
{
    if (SceneResponder.config.gt === undefined) 
    {
        return msgid;
    }
    
    return SceneResponder.config.gt.gettext(msgid);
}


SceneResponder.config.groups = 
{
    "VARIABLE": 0,
    "LIGHT": 1,
    "SHADOW": 2,
    "CLIMATE": 3,
    "AUDIO": 4,
    "VIDEO": 5,
    "SECURITY": 6,
    "ACCESS": 7,
    "JOKER": 8,
    "WHITE": 9,
    "DISPLAYS": 10
};

SceneResponder.config.scenes = 
{
    "DEC": {id: 11, name: _("Dim down")},
    "INC": {id: 12, name: _("Dim up")},
    "MIN": {id: 13, name: _("Off")},
    "MAX": {id: 14, name: _("On")},
    "STOP": {id: 15, name: _("Stop")},
    "T0_S0": {id: 0, name: _("Scene") + " 0"},
    "T0_S1": {id: 5, name: _("Scene") + " 1"},
    "T0_S2": {id: 17, name: _("Scene") + " 2"},
    "T0_S3": {id: 18, name: _("Scene") + " 3"},
    "T0_S4": {id: 19, name: _("Scene") + " 4"},
    "AUTO_STANDBY": {id: 40, name: _("Auto Standby")},
    "SIG_BELL": {id: 41, name: _("Bell")},
    "SIG_ALARM": {id: 42, name: _("Alarm")},
    "SIG_PANIC": {id: 43, name: _("Panic")},
    "STANDBY": {id: 48, name: _("Standby")},
    "DEEP_OFF": {id: 49, name: _("Deep Off")},
    "PRESENT": {id: 55, name: _("Present")},
    "ABSENT": {id: 56, name: _("Absent")},
    "PRIVATE": {id: 57, name: _("Private")},
    "SLEEPING": {id: 58, name: _("Sleeping")},
    "DAY": {id: 59, name: _("Day")},
    "NIGHT": {id: 60, name: _("Night")}
};

SceneResponder.config.zoneStore = new Ext.data.Store(
{
    sortInfo: 
    {
        field: 'id',
        direction: 'ASC'
    },
    reader: new Ext.data.JsonReader(
    {
        root: "zones",
        fields: [
        {
            name: 'id'
        }, 
        {
            name: 'name'
        }]
    })
});

Ext.Ajax.request(
{
    url: '/json/apartment/getStructure',
    disableCaching: true,
    timeout: 300000,
    method: "GET",
    scope: this,
    success: function(result, request)
    {
        try 
        {
            var jsonData = Ext.util.JSON.decode(result.responseText);
            if (jsonData.ok) 
            {
                var zones = [];
                Ext.each(jsonData.result.apartment.zones, function(zone)
                {
                    if (zone.id !== 0) 
                    {
                        zones.push(
                        {
                            id: zone.id,
                            name: zone.name ? zone.id + " - " +
                            zone.name : zone.id + " - " + _("Zone")
                        });
                    }
                });
                SceneResponder.config.zoneStore.loadData(
                {
                    zones: zones
                });
            }
        } 
        catch (err) 
        {
            Ext.MessageBox.alert(_("Error"), _("Could not load apartment structure") + ": \"" +
            err +
            "\"");
        }
    },
    failure: function(result, request)
    {
        Ext.MessageBox.alert(_("Error"), _("Could not load apartment structure") + ": \"" +
        request.url +
        "\"");
    }
    
});



SceneResponder.ScenePanel = Ext.extend(Ext.Panel, 
{

    constructor: function()
    {
        this.sceneStore = new Ext.data.ArrayStore(
        {
            fields: ['id', 'displayText']
        });
        
        for (var shortName in SceneResponder.config.scenes) 
        {
            var rec = new Ext.data.Record(
            {
                id: SceneResponder.config.scenes[shortName].id,
                displayText: SceneResponder.config.scenes[shortName].name
            });
            this.sceneStore.add(rec);
        };
		        
        this.sceneCombo = new Ext.form.ComboBox(
        {
            scope: this,
            editable: false,
            triggerAction: 'all',
            mode: 'local',
            ref: 'unit',
            fieldLabel: _("Scene"),
            store: this.sceneStore,
            displayField: 'displayText',
            hiddenField: 'id',
            valueField: 'id',
            value: SceneResponder.config.scenes.MAX.id,
            tooltip: _("choose the scene you want to observe"),
            width: 160
        });
        
        this.zoneCombo = new Ext.form.ComboBox(
        {
            xtype: 'combo',
            hideLabel: true,
            submitValue: false,
            // lazyInit: true,
            mode: 'local',
            forceSelection: true,
            store: SceneResponder.config.zoneStore,
            emptyText: _("Select zone") +
            "...",
            triggerAction: 'all',
            allowBlank: false,
            typeAhead: true,
            displayField: 'name',
            valueField: 'id',
            hiddenName: 'zone',
            width: 160
        });
        
        Ext.apply(this, 
        {
            bodyBorder: false,
            hideBorders: true,
            height: 50,
            scope: this,
            autoDestroy: true,
            layout: 'table',
            layoutConfig: 
            {
                columns: 3
            },
            items: [
            //row 1
            {
                xtype: 'panel',
                height: 20,
                bodyBorder: false,
                autoEl: 'span',
                cls: 'x-green-text black-text',
                html: _("Scene") + ":"
            }, 
            {
                xtype: 'box',
                height: 20,
                width: 10
            }, 
            {
                xtype: 'panel',
                height: 20,
                bodyBorder: false,
                autoEl: 'span',
                cls: 'x-green-text black-text',
                html: _("Zone") + ":"
            }, //row 2
 this.sceneCombo, 
            {
                xtype: 'box',
                height: 20,
                width: 10
            }, this.zoneCombo]
        });
        
        SceneResponder.ScenePanel.superclass.constructor.apply(this, arguments);
    }
    
});

Ext.reg('scenepanel', SceneResponder.ScenePanel);


SceneResponder.ActionPanel = Ext.extend(Ext.Panel, 
{
    constructor: function(config)
    {
        this.parentForm = config.parentForm;
        
        this.changingDesc = new Ext.form.Label(
        {
            scope: this,
            height: 20,
            bodyBorder: false,
            autoEl: 'span',
            cls: 'x-green-text black-text',
            text: _("Value") + ": " + "0%"
        });
        
        this.unitCombo = new Ext.form.ComboBox(
        {
            scope: this,
            editable: false,
            triggerAction: 'all',
            mode: 'local',
            ref: 'unit',
            width: 160,
            fieldLabel: _("Action"),
            forceSelection: true,
            submitValue: false,
            allowBlank: false,
            emptyText: _("Select unit") + "...",
            store: new Ext.data.ArrayStore(
            {
                fields: ['id', 'displayText'],
                data: [['group', _("Group")], ['scene', _("Scene")]]
            }),
            displayField: 'displayText',
            hiddenField: 'id',
            valueField: 'id',
            listeners: 
            {
                select: this.updateDesc.createDelegate(this)
            },
            value: 'group'
        });
        
        this.groupCombo = new Ext.form.ComboBox(
        {
            editable: false,
            mode: 'local',
            ref: 'group',
            width: 160,
            hideLabel: true,
            submitValue: false,
            forceSelection: true,
            triggerAction: 'all',
            allowBlank: false,
            store: new Ext.data.ArrayStore(
            {
                fields: ['id', 'displayText'],
                data: [[SceneResponder.config.groups.LIGHT, _("Light")]]
            }),
            displayField: 'displayText',
            valueField: 'id',
            value: SceneResponder.config.groups.LIGHT
        });
        
		this.sceneStore = new Ext.data.ArrayStore(
        {
            fields: ['id', 'displayText']
        });
        
        for (var shortName in SceneResponder.config.scenes) 
        {
            var rec = new Ext.data.Record(
            {
                id: SceneResponder.config.scenes[shortName].id,
                displayText: SceneResponder.config.scenes[shortName].name
            });
            this.sceneStore.add(rec);
        };
		
		this.sceneCombo = new Ext.form.ComboBox(
        {
            editable: false,
            hidden: false,
            mode: 'local',
            width: 160,
            hideLabel: true,
            submitValue: false,
            forceSelection: true,
            triggerAction: 'all',
            allowBlank: false,
            store: this.sceneStore,
            displayField: 'displayText',
            valueField: 'id',
            value: SceneResponder.config.scenes.MAX.id
        });
        
        this.value = new Ext.form.SliderField(
        {
            hideLabel: true,
            scope: this,
            width: 160,
            height: 20,
            minValue: 0,
            increment: 1,
            maxValue: 100,
            submitValue: false,
            tipText: function(thumb)
            {
                return String.format('{0}%', thumb.value);
            }
            
        });
        
        this.value.slider.on('changeComplete', function(slider, newVal, thumb)
        {
            this.changingDesc.setText(_("Value") + ": " + newVal + "%");
        }, this);
        
        this.zoneCombo = new Ext.form.ComboBox(
        {
            xtype: 'combo',
            hideLabel: true,
            submitValue: false,
            // lazyInit: true,
            mode: 'local',
            forceSelection: true,
            store: SceneResponder.config.zoneStore,
            emptyText: _("Select zone") +
            "...",
            triggerAction: 'all',
            allowBlank: false,
            typeAhead: true,
            displayField: 'name',
            valueField: 'id',
            hiddenName: 'zone',
            width: 160
        });
        
        this.scenePanel = new Ext.Panel(
        {
            xtype: 'panel',
            scope: this,
            width: 160,
            hidden: true,
            layout: 'form',
            bodyBorder: false,
            items: [this.sceneCombo]
        });
        
        this.valuePanel = new Ext.Panel(
        {
            xtype: 'panel',
            scope: this,
            width: 160,
            layout: 'form',
            bodyBorder: false,
            items: [this.value]
        });
        
        Ext.apply(this, 
        {
            layout: 'table',
            bodyBorder: false,
            hideBorders: true,
            height: 50,
            scope: this,
            autoDestroy: true,
            layoutConfig: 
            {
                columns: 10
            },
            items: [ // row 1
            {
                xtype: 'box',
                height: 20,
                width: 19
            }, 
            {
                xtype: 'panel',
                height: 20,
                bodyBorder: false,
                autoEl: 'span',
                cls: 'x-green-text black-text',
                html: _("Unit") + ":"
            }, 
            {
                xtype: 'box',
                height: 20,
                width: 10
            }, 
            {
                xtype: 'panel',
                height: 20,
                bodyBorder: false,
                autoEl: 'span',
                cls: 'x-green-text black-text',
                html: _("Zone") + ":"
            }, 
            {
                xtype: 'box',
                height: 20,
                width: 10
            }, 
            {
                xtype: 'panel',
                height: 20,
                bodyBorder: false,
                autoEl: 'span',
                cls: 'x-green-text black-text',
                html: _("Group") + ":"
            }, 
            {
                xtype: 'box',
                height: 20,
                width: 10
            }, this.changingDesc, 
            {
                xtype: 'box',
                height: 20,
                width: 10
            }, 
            {
                xtype: 'box',
                height: 20,
                width: 10
            }, // row 2
             {
                xtype: 'panel',
                layout: 'form',
                bodyBorder: false,
                colspan: 2,
                items: [this.unitCombo]
            }, 
            {
                xtype: 'box',
                height: 20,
                width: 10
            }, 
            {
                xtype: 'panel',
                layout: 'form',
                bodyBorder: false,
                items: [this.zoneCombo]
            }, 
            {
                xtype: 'box',
                height: 20,
                width: 10
            }, 
            {
                xtype: 'panel',
                layout: 'form',
                bodyBorder: false,
                items: [this.groupCombo]
            }, 
            {
                xtype: 'box',
                height: 20,
                width: 10
            }, 
            {
                xtype: 'panel',
                bodyBorder: false,
                layout: 'fit',
                items: [this.valuePanel, this.scenePanel]
            }, 
            {
                xtype: 'box',
                height: 20,
                width: 10
            }, 
            {
                xtype: 'button',
                text: _("Remove"),
                tooltip: _("Remove this action"),
                scope: this,
                handler: function()
                {
                    this.parentForm.removeAction(this);
                }
                
            }]
        });
        
        SceneResponder.ActionPanel.superclass.constructor.apply(this, arguments);
    },
    updateDesc: function(combo, record, index)
    {
        if (record.get('id') === 'scene') 
        {
            this.changingDesc.setText(_("Scene") + ":");
            this.valuePanel.setVisible(false);
            this.scenePanel.setVisible(true);
        }
        else 
        {
            this.changingDesc.setText(_("Value") + ": " +
            this.value.getValue() +
            "%");
            this.scenePanel.setVisible(false);
            this.valuePanel.setVisible(true);
        }
    },
    
    setAction: function(action)
    {
        if (action.unit == 'group') 
        {
            this.unitCombo.setValue(action.unit);
            this.groupCombo.setValue(action.group);
            this.zoneCombo.setValue(action.zone);
            
            var val = action.value / 255;
            val = val * 100;
            if (val < 0) 
            {
                val = 0;
            }
            else if (val > 100) 
            {
                val = 100;
            }
            this.value.setValue(val);
            
            this.changingDesc.setText(_("Value") + ": " +
            this.value.getValue() +
            "%");
            this.scenePanel.setVisible(false);
            this.valuePanel.setVisible(true);
        }
        else if (action.unit == 'scene') 
        {
            this.unitCombo.setValue(action.unit);
            this.groupCombo.setValue(action.group);
            this.zoneCombo.setValue(action.zone);
            this.sceneCombo.setValue(action.scene);
            
            this.valuePanel.setVisible(false);
            this.scenePanel.setVisible(true);
            this.changingDesc.setText(_("Scene") + ":");
        }
    }
    
});

Ext.reg('actionpanel', SceneResponder.ActionPanel);




SceneResponder.ActionForm = Ext.extend(Ext.FormPanel, 
{
    initComponent: function()
    {
        this.actionCount = 0;
        this.dateCount = 0;
        
        this.timeCombo = new Ext.form.ComboBox(
        {
            scope: this,
            editable: true,
            triggerAction: 'all',
            mode: 'local',
            ref: 'unit',
            fieldLabel: _("Time (in minutes)"),
            store: new Ext.data.ArrayStore(
            {
                fields: ['id', 'displayText'],
                data: [['0', _("0")], ['1', _("1")], ['2', _("2")], ['3', _("3")], ['4', _("4")], ['5', _("5")], ['10', _("10")], ['15', _("15")], ['30', _("30")], ['45', _("45")], ['60', _("60")]]
            }),
            displayField: 'displayText',
            hiddenField: 'id',
            valueField: 'id',
            value: '5',
            tooltip: _("choose the time between the scene call and the configured actions")
        
        });
        
        this.scenePanel = new SceneResponder.ScenePanel();
        
        NameField = Ext.extend(Ext.form.TextField, 
        {
            initComponent: function()
            {
                Ext.apply(this, 
                {
                    xtype: 'textfield',
                    width: 330,
                    ref: 'SceneResponderName',
                    allowBlank: false,
                    fieldLabel: _("Name"),
                    padding: 6
                });
                NameField.superclass.initComponent.apply(this, arguments);
            },
            
            validate: function()
            {
                try 
                {
                    var store = this.findParentByType('actionform').findParentByType('window').mainpanel.actionview.actionStore;
                    for (var i = 0; i < store.getCount(); i++) 
                    {
                        if (store.getAt(i).get('name') == this.getValue()) 
                        {
                            return false;
                        }
                    }
                } 
                catch (error) 
                {
                    Ext.MessageBox.alert(error);
                    return false;
                }
                return true;
                
            }
            
        });
        
        this.nameField = new NameField();
        
        Ext.apply(this, 
        {
            autoScroll: true,
            bodyBorder: false,
            hideBorders: true,
            items: [
            {
                xtype: 'panel',
                padding: 6,
                bodyBorder: false,
                layout: 'form',
                scope: this,
                autoEl: 'span',
                ref: 'mainPanel',
                items: [
                {
                    xtype: 'fieldset',
                    scope: this,
                    ref: 'name',
                    title: _("Name"),
                    items: [this.nameField]
                }, 
                {
                    xtype: 'fieldset',
                    title: _("Trigger"),
                    ref: 'scene',
                    bodyBorder: false,
                    // layout: 'fit',
                    // height: 200,
                    autoScroll: true,
                    scope: this,
                    items: [this.scenePanel]
                }, 
                {
                    xtype: 'fieldset',
                    title: _("Automatic actions"),
                    bodyBorder: false,
                    // layout: 'fit',
                    // height: 200,
                    ref: 'actionHolder',
                    autoScroll: true,
                    scope: this,
                    items: [
                    {
                        xtype: 'button',
                        
                        text: _("Add action"),
                        tooltip: _("Add a new action, that is executed after the configured time when the triggering scene call is overservated"),
                        scope: this,
                        handler: function()
                        {
                            this.mainPanel.actionHolder.add(new SceneResponder.ActionPanel(
                            {
                                parentForm: this
                            }));
                            this.mainPanel.actionHolder.doLayout();
                            this.actionCount++;
                        }
                        
                    }]
                }, // actionHolder panel
                 {
                    xtype: 'fieldset',
                    title: _("Time"),
                    bodyBorder: false,
                    // layout: 'fit',
                    // height: 200,
                    autoScroll: true,
                    scope: this,
                    items: [this.timeCombo]
                }]
            }]
        });
        
        SceneResponder.ActionForm.superclass.initComponent.apply(this, arguments);
    },
    getTrigger: function()
    {
        var trigger = {};
        trigger.scene = this.scenePanel.sceneCombo.getValue();
        trigger.zone = this.scenePanel.zoneCombo.getValue();
        return trigger;
    },
    
    getDuration: function()
    {
        return this.timeCombo.getValue();
    },
    
    removeAction: function(action)
    {
        this.mainPanel.actionHolder.remove(action);
        this.mainPanel.actionHolder.doLayout();
        this.actionCount--;
        if (this.actionCount < 0) 
        {
            this.actionCount = 0;
        }
    },
    getActionData: function()
    {
        // process actions
        var counter = 0;
        var actions = {};
        var actionParameters = [];
        var action;
        do 
        {
            action = this.mainPanel.actionHolder.get(counter);
            counter++;
            
            if (action === undefined) 
            {
                break;
            }
            
            if (action.getXType() != 'actionpanel') 
            {
                continue;
            }
            
            
            var unit = action.unitCombo.getValue();
            var group = action.groupCombo.getValue();
            var zone = action.zoneCombo.getValue();
            
            var params = {};
            params.unit = unit;
            params.group = group;
            params.zone = zone;
            
            if (unit === 'group') 
            {
                var val = action.value.getValue() / 100;
                val = val * 255;
                if (val < 0) 
                {
                    val = 0;
                }
                else if (val > 255) 
                {
                    val = 255;
                }
                
                params.value = Math.round(val);
            }
            else if (unit === 'scene') 
            {
                params.scene = action.sceneCombo.getValue();
            }
            
            actionParameters.push(params);
            
        }
        while (action !== undefined);
        
        actions.actions = actionParameters;
        return actions;
    },
    
    setParameters: function(name, trigger, actions, duration)
    {
        this.mainPanel.name.SceneResponderName.setValue(name);
        trigger = Ext.util.JSON.decode(trigger);
        this.scenePanel.sceneCombo.setValue(trigger.scene);
        this.scenePanel.zoneCombo.setValue(trigger.zone);
        this.timeCombo.setValue(duration);
        
        for (var i = 0; i < actions.actions.length; i++) 
        {
            var actionPanel = new SceneResponder.ActionPanel(
            {
                parentForm: this
            });
            actionPanel.setAction(actions.actions[i]);
            this.mainPanel.actionHolder.add(actionPanel);
            this.actionCount++;
        }
        this.mainPanel.actionHolder.doLayout();
    }
    
});

Ext.reg('actionform', SceneResponder.ActionForm);

SceneResponder.ActionView = Ext.extend(Ext.Panel, 
{

    initComponent: function()
    {
        this.actionStore = new Ext.data.ArrayStore(
        {
            fields: ['name', 'trigger', 'action', 'duration']
        });
        
        this.ActionRecord = Ext.data.Record.create([
        {
            name: "name"
        }, 
        {
            trigger: "trigger"
        }, 
        {
            action: "action"
        }, 
        {
            duration: "duration"
        }]);
        
        this.selectionModel = new Ext.grid.RowSelectionModel(
        {
            singleSelect: true
        });
        
        this.actionGrid = new Ext.grid.GridPanel(
        {
            store: this.actionStore,
            columns: [
            {
                id: 'name',
                header: _("Name"),
                dataIndex: 'name',
                sortable: true
            }, 
            {
                id: 'trigger',
                header: _("Trigger"),
                dataIndex: 'trigger',
                sortable: true
            }, 
            {
                id: 'action',
                header: _("Action"),
                dataIndex: 'action',
                sortable: true
            }, 
            {
                id: 'duration',
                header: _("Duration"),
                dataIndex: 'duration',
                sortable: true
            }],
            trackMouseOver: false,
            viewConfig: 
            {
                forceFit: true
            },
            sm: this.selectionModel,
            bodyBorder: false
        });
        
        Ext.apply(this, 
        {
            layout: 'fit',
            items: [this.actionGrid]
        });
        
        SceneResponder.ActionPanel.superclass.initComponent.apply(this, arguments);
        
        //load stored actions from property tree
        Ext.Ajax.request(
        {
            url: '/json/property/getChildren?path=/scripts/scene-responder/triggers',
            disableCaching: true,
            timeout: 300000,
            method: "GET",
            scope: this,
            success: function(result, request)
            {
                try 
                {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) 
                    {
                        var children = jsonData.result;
                        for (var i = 0; i < children.length; i++) 
                        {
                            Ext.Ajax.request(
                            {
                                url: '/json/property/getString?path=/scripts/scene-responder/triggers/' + children[i].name + '/params',
                                disableCaching: true,
                                timeout: 300000,
                                method: "GET",
                                scope: this,
                                success: function(result, request)
                                {
                                    try 
                                    {
                                        var jsonData = Ext.util.JSON.decode(result.responseText);
                                        if (jsonData.ok) 
                                        {
                                            //Ext.MessageBox.alert('json', jsonData.result.value);
                                            var params = Ext.util.JSON.decode(jsonData.result.value);
                                            this.addSceneResponder(params.name, Ext.util.JSON.decode(params.trigger), Ext.util.JSON.decode(params.actions), params.duration);
                                        }
                                    } 
                                    catch (err) 
                                    {
                                        Ext.MessageBox.alert(_("Error"), _("Could not load trigger params") + ": \"" +
                                        err +
                                        "\"");
                                    }
                                },
                                failure: function(result, request)
                                {
                                    Ext.MessageBox.alert(_("Error"), _("Could not load trigger params") + ": \"" +
                                    request.url +
                                    "\"");
                                }
                                
                            });
                        }
                    }
                } 
                catch (err) 
                {
                    Ext.MessageBox.alert(_("Error"), _("Could not load enabled triggers") + ": \"" +
                    err +
                    "\"");
                }
            },
            failure: function(result, request)
            {
                Ext.MessageBox.alert(_("Error"), _("Could not load enabled triggers") + ": \"" +
                request.url +
                "\"");
            }
            
        });
        
    },
    
    
    
    addSceneResponder: function(name, trigger, actions, duration)
    {
        var actionString = '';
        for (i = 0; i < actions.actions.length; i++) 
        {
            var action = actions.actions[i];
            if (action === undefined) 
            {
                Ext.MessageBox.alert("error", "undefined");
                break;
            }
            
            if (action.unit === 'scene') 
            {
                //search for scene number + name
                for (var shortName in SceneResponder.config.scenes) 
                {
                    if (SceneResponder.config.scenes[shortName].id == action.scene) 
                    {
                        actionString = actionString + _("Scene") + ' ' + SceneResponder.config.scenes[shortName].name + ' ' + _("in Zone") + ' ' + action.zone + '\n';
                        break;
                    }
                }
            }
            else if (action.unit === 'group') 
            {
                actionString = actionString + _("Value") + ' ' + action.value + ' ' + _("in Zone") + ' ' + action.zone + '\n';
            }
        }
        
        //search for scene number + name
        var triggerString = "";
        for (var shortName in SceneResponder.config.scenes) 
        {
            if (SceneResponder.config.scenes[shortName].id == trigger.scene) 
            {
                triggerString = _("Scene") + ' ' + SceneResponder.config.scenes[shortName].name + ' ' + _("in Zone") + ' ' + trigger.zone;
                break;
            }
        }
        
        var rec = new this.ActionRecord(
        {
            name: name,
            trigger: triggerString,
            action: actionString,
            duration: duration
        });
        this.actionStore.add(rec);
    },
    
    getSelectedName: function()
    {
        try 
        {
            return this.selectionModel.getSelected().get('name');
        } 
        catch (e) 
        {
            Ext.MessageBox.alert(_("Error"), _("please create at least one action before deleting or modifying"));
        }
    },
    
    getSelectedID: function()
    {
        try 
        {
            var rec = this.selectionModel.getSelected();
            return this.actionStore.indexOf(rec);
        } 
        catch (e) 
        {
            Ext.MessageBox.alert(_("Error"), _("please create at least one action before deleting or modifying"));
        }
    },
    
    removeSelected: function()
    {
        this.actionStore.remove(this.selectionModel.getSelected());
    }
    
});

Ext.reg('actionview', SceneResponder.ActionView);


SceneResponder.MainPanel = Ext.extend(Ext.Panel, 
{
    initComponent: function()
    {
        Ext.apply(this, 
        {
            layout: 'border',
            title: _("Configure action"),
            items: [
            {
                xtype: 'panel',
                //region: 'north',
                autoEl: 'span',
                cls: 'x-green-text black-text',
                padding: 4,
                bodyBorder: false,
                html: _("This add-on allows you to configure actions.")
            }, 
            {
                xtype: 'actionview',
                ref: 'actionview',
                region: 'center'
            }],
            bbar: 
            {
                xtype: 'toolbar',
                items: [
                {
                    xtype: 'tbtext',
                    cls: 'x-black-text x-text-align-left x-text-cut',
                    text: _("Version") + ': ' + SceneResponder.config.VERSION
                }, 
                {
                    xtype: 'tbseparator',
                    width: 8
                }, '->', 
                {
                    xtype: 'button',
                    text: _("Delete selected action"),
                    ref: 'deleteSceneResponderButton',
                    iconCls: 'icon-delete-action',
                    handler: function()
                    {
                        Ext.Ajax.request(
                        {
                            url: '/json/event/raise',
                            scope: this,
                            params: 
                            {
                                name: 'sceneresponder_delete',
                                parameter: 'scenerespondername=' + this.findParentByType('scenerespondermainpanel').actionview.getSelectedName()
                            },
                            //url: '/json/property/remove?path=/scripts/scene-responder/triggers/' + this.findParentByType('scenerespondermainpanel').actionview.getSelectedName(),
                            method: 'GET',
                            scope: this,
                            success: function(action)
                            {
                                try 
                                {
                                    var jsonData = Ext.util.JSON.decode(action.responseText);
                                    if (jsonData.ok) 
                                    {
                                        this.findParentByType('scenerespondermainpanel').actionview.removeSelected();
                                    }
                                    else 
                                    {
                                        Ext.Msg.alert(_("Error"), _("Failed to delete action"));
                                    }
                                } 
                                catch (error) 
                                {
                                    Ext.Msg.alert(_("Error"), _("Failed to delete action"));
                                }
                            },
                            failure: function(action)
                            {
                                Ext.Msg.alert(_("Error"), _("Failed to delete action, please check if dSS is still running"));
                            }
                            
                        });
                    }
                    
                }, 
                {
                    xtype: 'button',
                    text: _("Modify selected action"),
                    ref: 'modifySceneResponderButton',
                    iconCls: 'icon-edit-action',
                    handler: function()
                    {
                        this.findParentByType('scenerespondermainpanel').actionview.getSelectedName(); //check if something is selected
                        Ext.Ajax.request(
                        {
                            url: '/json/property/getString?path=/scripts/scene-responder/triggers/' + this.findParentByType('scenerespondermainpanel').actionview.getSelectedName() + '/params',
                            method: "GET",
                            scope: this,
                            success: function(result, request)
                            {
                                try 
                                {
                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                    if (jsonData.ok) 
                                    {
                                        var params = Ext.util.JSON.decode(jsonData.result.value);
                                        //delete existing event
                                        Ext.Ajax.request(
                                        {
                                            url: '/json/event/raise',
                                            params: 
                                            {
                                                name: 'sceneresponder_delete',
                                                parameter: 'scenerespondername=' + this.findParentByType('scenerespondermainpanel').actionview.getSelectedName()
                                            },
                                            //url: '/json/property/remove?path=/scripts/scene-responder/triggers/' + this.findParentByType('scenerespondermainpanel').actionview.getSelectedName(),
                                            method: 'GET',
                                            scope: this,
                                            success: function(action)
                                            {
                                                try 
                                                {
                                                    var jsonData = Ext.util.JSON.decode(action.responseText);
                                                    if (jsonData.ok) 
                                                    {
                                                        this.findParentByType('scenerespondermainpanel').actionview.removeSelected();
                                                    }
                                                    else 
                                                    {
                                                        Ext.Msg.alert(_("Error"), _("Failed to modify action"));
                                                    }
                                                } 
                                                catch (error) 
                                                {
                                                    Ext.Msg.alert(_("Error"), _("Failed to modify action"));
                                                }
                                            },
                                            failure: function(action)
                                            {
                                                Ext.Msg.alert(_("Error"), _("Failed to modify action, please check if dSS is still running"));
                                            }
                                            
                                        });
                                        
                                        //open form to modify existing data
                                        var win = new Ext.Window(
                                        {
                                            title: _("Configure action"),
                                            layout: 'fit',
                                            hideBorders: true,
                                            height: 500,
                                            width: 900,
                                            minHeight: 400,
                                            minWidth: 800,
                                            autoScroll: true,
                                            scope: this,
                                            mainpanel: this.findParentByType('scenerespondermainpanel'),
                                            items: [
                                            {
                                                xtype: 'actionform',
                                                ref: 'actionForm'
                                            }],
                                            buttons: [
                                            {
                                                text: _("Save and activate"),
                                                scope: this,
                                                handler: function()
                                                {
                                                    win.activateSceneResponder();
                                                }
                                                
                                            }, 
                                            {
                                                text: _("Delete"),
                                                scope: this,
                                                handler: function()
                                                {
                                                    win.close();
                                                    this.enable();
                                                    this.findParentByType('scenerespondermainpanel').el.unmask();
                                                }
                                                
                                            }],
                                            
                                            setParameters: function(name, trigger, actions, duration)
                                            {
                                                this.actionForm.setParameters(name, trigger, actions, duration);
                                            },
                                            
                                            activateSceneResponder: function()
                                            {
                                                if (!this.actionForm.nameField.validate()) 
                                                {
                                                    Ext.MessageBox.alert(_("Error"), _("Please choose a unique name"));
                                                    return;
                                                }
                                                
                                                if (!this.actionForm.getForm().isValid()) 
                                                {
                                                    Ext.MessageBox.alert(_("Error"), _("Please fill out all mandatory fields"));
                                                    return;
                                                }
                                                
                                                if (this.actionForm.actionCount === 0) 
                                                {
                                                    Ext.MessageBox.alert(_("Error"), _("Please add at least one action"));
                                                    return;
                                                }
                                                
                                                
                                                var name = this.actionForm.mainPanel.name.SceneResponderName.getValue();
                                                var scenerespondername = 'scenerespondername=' + name + ';';
                                                
                                                var trigger = this.actionForm.getTrigger();
                                                var scenerespondertrigger = 'scenerespondertrigger=' + Ext.util.JSON.encode(trigger) + ';';
                                                
                                                var duration = this.actionForm.getDuration();
                                                var sceneresponderduration = 'sceneresponderduration=' + duration + ';';
                                                
                                                var actions = this.actionForm.getActionData();
                                                
                                                var sceneresponderactions = 'sceneresponderactions=' + Ext.util.JSON.encode(actions);
                                                
                                                Ext.Ajax.request(
                                                {
                                                    url: '/json/event/raise',
                                                    method: 'GET',
                                                    scope: this,
                                                    params: 
                                                    {
                                                        name: 'sceneresponder_add',
                                                        parameter: scenerespondername + scenerespondertrigger + sceneresponderduration + sceneresponderactions
                                                    },
                                                    success: function(action)
                                                    {
                                                        try 
                                                        {
                                                            var jsonData = Ext.util.JSON.decode(action.responseText);
                                                            if (jsonData.ok) 
                                                            {
                                                                this.mainpanel.actionview.addSceneResponder(name, trigger, actions, duration);
                                                                this.close();
                                                                return;
                                                            }
                                                            else 
                                                            {
                                                                Ext.Msg.alert(_("Error"), _("Failed to add action"));
                                                            }
                                                        } 
                                                        catch (error) 
                                                        {
                                                            Ext.Msg.alert(_("Error"), _("Failed to add action"));
                                                        }
                                                    },
                                                    failure: function(action)
                                                    {
                                                        Ext.Msg.alert(_("Error"), _("Failed to add action, please check if dSS is still running"));
                                                    }
                                                    
                                                });
                                            }
                                            
                                            // all ok, submit form 
                                        });
                                        
                                        win.on('beforedestroy', function()
                                        {
                                            this.findParentByType('scenerespondermainpanel').el.unmask();
                                            this.enable();
                                        }, this);
                                        
                                        
                                        win.setParameters(params.name, params.trigger, Ext.util.JSON.decode(params.actions), params.duration);
                                        
                                        this.disable();
                                        this.findParentByType('scenerespondermainpanel').el.mask();
                                        win.show(this);
                                    }
                                } 
                                
                                catch (err) 
                                {
                                    Ext.MessageBox.alert(_("Error"), _("Could not load trigger params") + ": \"" +
                                    err +
                                    "\"");
                                }
                            },
                            failure: function(result, request)
                            {
                                Ext.MessageBox.alert(_("Error"), _("Could not load trigger params") + ": \"" +
                                request.url +
                                "\"");
                            }
                            
                        });
                    }
                    
                }, 
                {
                    xtype: 'button',
                    text: _("Add new action"),
                    ref: 'addSceneResponderButton',
                    iconCls: 'icon-add-action',
                    handler: function()
                    {
                        var win = new Ext.Window(
                        {
                            title: _("Configure action"),
                            layout: 'fit',
                            hideBorders: true,
                            height: 500,
                            width: 900,
                            minHeight: 400,
                            minWidth: 800,
                            autoScroll: true,
                            scope: this,
                            mainpanel: this.findParentByType('scenerespondermainpanel'),
                            items: [
                            {
                                xtype: 'actionform',
                                ref: 'actionForm'
                            }],
                            buttons: [
                            {
                                text: _("Save and activate"),
                                scope: this,
                                handler: function()
                                {
                                    win.activateSceneResponder();
                                }
                                
                            }, 
                            {
                                text: _("Cancel"),
                                scope: this,
                                handler: function()
                                {
                                    win.close();
                                    this.enable();
                                    this.findParentByType('scenerespondermainpanel').el.unmask();
                                }
                                
                            }],
                            
                            activateSceneResponder: function()
                            {
                                if (!this.actionForm.nameField.validate()) 
                                {
                                    Ext.MessageBox.alert(_("Error"), _("Please choose a unique name"));
                                    return;
                                }
                                
                                if (!this.actionForm.getForm().isValid()) 
                                {
                                    Ext.MessageBox.alert(_("Error"), _("Please fill out all mandatory fields"));
                                    return;
                                }
                                
                                if (this.actionForm.actionCount === 0) 
                                {
                                    Ext.MessageBox.alert(_("Error"), _("Please add at least one action"));
                                    return;
                                }
                                
                                
                                var name = this.actionForm.mainPanel.name.SceneResponderName.getValue();
                                var scenerespondername = 'scenerespondername=' + name + ';';
                                
                                var trigger = this.actionForm.getTrigger();
                                var scenerespondertrigger = 'scenerespondertrigger=' + Ext.util.JSON.encode(trigger) + ';';
                                
                                var duration = this.actionForm.getDuration();
                                var sceneresponderduration = 'sceneresponderduration=' + duration + ';';
                                
                                var actions = this.actionForm.getActionData();
                                var sceneresponderactions = 'sceneresponderactions=' + Ext.util.JSON.encode(actions);
                                
                                Ext.Ajax.request(
                                {
                                    url: '/json/event/raise',
                                    method: 'GET',
                                    scope: this,
                                    params: 
                                    {
                                        name: 'sceneresponder_add',
                                        parameter: scenerespondername + scenerespondertrigger + sceneresponderduration + sceneresponderactions
                                    },
                                    success: function(action)
                                    {
                                        try 
                                        {
                                            var jsonData = Ext.util.JSON.decode(action.responseText);
                                            if (jsonData.ok) 
                                            {
                                                this.mainpanel.actionview.addSceneResponder(name, trigger, actions, duration);
                                                this.close();
                                                return;
                                            }
                                            else 
                                            {
                                                Ext.Msg.alert(_("Error"), _("Failed to add action"));
                                            }
                                        } 
                                        catch (error) 
                                        {
                                            Ext.Msg.alert(_("Error"), _("Failed to add action"));
                                        }
                                    },
                                    failure: function(action)
                                    {
                                        Ext.Msg.alert(_("Error"), _("Failed to add action, please check if dSS is still running"));
                                    }
                                    
                                });
                            }
                            
                            // all ok, submit form 
                        });
                        
                        win.on('beforedestroy', function()
                        {
                            this.findParentByType('scenerespondermainpanel').el.unmask();
                            this.enable();
                        }, this);
                        
                        this.disable();
                        this.findParentByType('scenerespondermainpanel').el.mask();
                        win.show(this);
                    }
                    
                }]
            }
        
        });
        
        SceneResponder.MainPanel.superclass.initComponent.apply(this, arguments);
    }
    
});

Ext.reg('scenerespondermainpanel', SceneResponder.MainPanel);

Ext.onReady(function()
{
    Ext.get('start').remove();
    
    Ext.Ajax.request(
    {
        url: '/json/system/login',
        params: 
        {
            user: 'dssadmin',
            password: 'dssadmin'
        },
        scope: this,
        method: 'GET',
        disableCaching: true,
        success: function(response, options)
        {
            try 
            {
                var jsonData = Ext.util.JSON.decode(response.responseText);
                if (jsonData.ok) 
                {
                    this.viewport = new Ext.Viewport(
                    {
                        hideBorders: true,
                        layout: 'border',
                        items: [
                        {
                            xtype: 'box',
                            region: 'north',
                            height: 80,
                            autoEl: 
                            {
                                tag: 'div',
                                html: '<a href="http://digitalstrom.org/" alt="' +
                                _("visit digitalSTROM.org") +
                                '"><img id="logo" src="images/ds_logo.gif"' +
                                'alt="dS-Logo"></a>' +
                                '<h1><a href="http://digitalstrom.org/"' +
                                ' alt="visit digitalSTROM.org">' +
                                _("digitalSTROM Server - Scene Responder") +
                                '</a></h1>'
                            }
                        }, 
                        {
                            region: 'center',
                            xtype: 'scenerespondermainpanel'
                        }]
                    });
                    document.title = _("digitalSTROM Server - Scene Responder");
                }
                else 
                    Ext.MessageBox.alert(_('Error'), _('Could not log in') + ': ' + jsonData.message);
            } 
            catch (err) 
            {
                Ext.MessageBox.alert(_('Error'), _('Could not log in') + ': ' + err);
            }
        },
        failure: function(response, options)
        {
            Ext.MessageBox.alert(_('Error'), _('Could not send login command to dSS'));
        }
        
    });
});
